import Flutter
import UIKit
import NearbyMessages

fileprivate enum ErrorCodes: String {
    case WRONG_ARGS = "WRONG_ARGS"
    case BAD_ARGS = "BAD_ARGS"
    case API_NOT_SET = "API_NOT_SET"
    
    var value: String {
        return self.rawValue
    }
    
    var message: String {
        switch self {
        case .BAD_ARGS:
            return "Arguments must be String"
        case .API_NOT_SET:
            return "Api key isn't set"
        case .WRONG_ARGS:
            return "Wrong argument"
        }
    }
}

public class SwiftFlutterNearbyMessagesApiPlugin: NSObject, FlutterPlugin {
    private var client: GNSMessageManager? = nil
    private var channel: FlutterMethodChannel!
    private var subscription: GNSSubscription? = nil
    private var publication: GNSPublication? = nil
    
    private let _gNSStrategy: GNSStrategy = {
        return GNSStrategy(paramsBlock: { strategyParams in
            strategyParams?.allowInBackground = false
            strategyParams?.discoveryMediums = .default
            strategyParams?.discoveryMode = .default
        })
    }()
    
    public static func register(with registrar: FlutterPluginRegistrar) {
        let channel = FlutterMethodChannel(name: "flutter_nearby_messages_api", binaryMessenger: registrar.messenger())
        let instance = SwiftFlutterNearbyMessagesApiPlugin()
        instance.channel = channel
        registrar.addMethodCallDelegate(instance, channel: channel)
    }
    
    public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        switch call.method {
        case "setAPIKey":
            self.initGNSMessageManager(call, result: result)
        case "backgroundSubscribe":
            self.backgroundSubscribe(call, result: result)
        case "backgroundUnsubscribe":
            self.backgroundUnsubscribe(call, result: result)
        case "publish":
            self.publish(call, result: result)
        case "unPublish":
            self.unPublish(call, result: result)
        case "setGranted":
            self.setGranted(call, result: result)
        case "enableDebugMode":
            self.enableDebugMode(call, result: result)
        case "disableDebugMode":
            self.disableDebugMode(call, result: result)
        default: result(FlutterMethodNotImplemented)
        }
    }
    
    private func initGNSMessageManager(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        if let arguments = call.arguments as? [String : Any],
            let apiKey = arguments["apikey"] as? String {
            client = GNSMessageManager(apiKey: apiKey)
            result(nil)
        } else {
            result(FlutterError(code: ErrorCodes.WRONG_ARGS.value, message: ErrorCodes.WRONG_ARGS.message, details: nil))
        }
    }
    
    private func backgroundSubscribe(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        guard let _ = self.client else {
            result(FlutterError(code: ErrorCodes.API_NOT_SET.value, message: ErrorCodes.API_NOT_SET.message, details: nil))
            return
        }
        publication = nil
        
        subscription = self.client!.subscription(messageFoundHandler: { (msg: GNSMessage?) in
            if let msg = msg {
                self.channel.invokeMethod("onFound", arguments: String(data: msg.content, encoding: .utf8))
            }
        }, messageLostHandler: { (msg: GNSMessage?) in
            if let msg = msg {
                self.channel.invokeMethod("onLost", arguments: String(data: msg.content, encoding: .utf8))
            }
        }, paramsBlock: { params in
            params?.strategy = self._gNSStrategy
        })
        result(nil)
    }
    
    private func backgroundUnsubscribe(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        subscription = nil
        result(nil)
    }
    
    private func setGranted(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        if let arguments = call.arguments as? [String: Any],
            let isGranted = arguments["isGranted"] as? Bool {
            GNSPermission.setGranted(isGranted)
        }
        result(nil)
    }
    
    private func publish(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        guard let _ = self.client else {
            result(FlutterError(code: ErrorCodes.API_NOT_SET.value, message: ErrorCodes.API_NOT_SET.message, details: nil))
            return
        }
        subscription = nil
        
        if let arguments = call.arguments as? [String: Any],
            let msg = arguments["message"] as? String {
            let message = GNSMessage(content: msg.data(using: .utf8))
            publication = self.client!.publication(with: message, paramsBlock: { (params) in
                params?.strategy = self._gNSStrategy
            })
            result(nil)
        } else {
            result(FlutterError(code: ErrorCodes.BAD_ARGS.value, message: ErrorCodes.BAD_ARGS.message, details: nil))
        }
    }
    
    private func unPublish(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        self.publication = nil
        result(nil)
    }
    
    private func enableDebugMode(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        GNSMessageManager.setDebugLoggingEnabled(true)
        result(nil)
    }
    
    private func disableDebugMode(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        GNSMessageManager.setDebugLoggingEnabled(false)
        result(nil)
    }
}
